@@ -1,28 +1,35 @@
-#' Run forward selection
+##=============================================================================
+##
+## Copyright (c) 2013-2019 Marco Colombo
+##
+## This program is free software: you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published by
+## the Free Software Foundation, either version 2 of the License, or
+## (at your option) any later version.
+##
+## This program is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
+##
+## You should have received a copy of the GNU General Public License
+## along with this program.  If not, see <http://www.gnu.org/licenses/>.
+##
+##=============================================================================
+
+
+#' Cross-validated forward selection
 #'
-#' Run forward selection starting from a set of variables or a model.
+#' Run forward selection starting from a baseline model. As it uses
+#' all observations in the `data` dataframe, it is not possible to
+#' produce unbiased estimates of the predictive performance of the panel
+#' selected (use [nested.fs()] for that purpose).
 #'
 #' At each iteration, this function runs cross-validation to choose which
-#' variable enters the final panel by fitting the current model (defined in
-#' \code{init.model} as a list of variables or a formula for the first
-#' iteration) augmented by one of the remaining variables at a time.
+#' variable enters the final panel by fitting the current model augmented by
+#' each remaining variable considered one at a time.
 #'
-#' This function is used to choose a panel of variables on the data. As it uses
-#' all observations in the \code{x} dataframe, it is not possible to
-#' produce unbiased estimates of the predictive performance of the panel
-#' selected (use \code{\link{nested.forward.selection}} for that purpose).
-#'
-#' In the case of a binary outcome when very large number of predictors is
-#' available, it may be convenient to apply a univariate association filter.
-#' If \code{num.filter} is set to a positive value, then all available
-#' predictors (excluding those whose name is matched by \code{filter.ignore})
-#' are tested for univariate association with the outcome, and only the first
-#' \code{num.filter} enter the selection phase, while the others are filtered
-#' out. This is done on the training part of all inner folds. Filtering can
-#' enhance the performance of forward selection when the number of available
-#' variables exceeds about 30-40.
-#'
-#' By default variables are selected according to the \code{paired.test}
+#' By default variables are selected according to the `paired.test`
 #' criterion. At each iteration, the sampling distribution of differences in
 #' validation log-likelihood obtained across all inner cross-validation folds
 #' of the models with and without each additional variable are tested against
@@ -33,89 +40,83 @@
 #' including the marker will not decrease the validation log-likelihood
 #' (approximate false discovery rate).
 #'
-#' @template args-forward
-#' @template args-outcome
+#' In the case of a binary outcome when very large number of predictors is
+#' available, it may be convenient to apply a univariate association filter.
+#' If `num.filter` is set to a positive value, then all available
+#' predictors (excluding those whose name is matched by `filter.ignore`)
+#' are tested for univariate association with the outcome, and only the first
+#' `num.filter` enter the selection phase, while the others are filtered
+#' out. This is done on the training part of all inner folds. Filtering can
+#' enhance the performance of forward selection when the number of available
+#' variables exceeds about 30-40.
+#'
+#' @template args-interface
 #' @template args-family
 #' @param choose.from Indices or variable names over which the selection should
-#'        be performed. If \code{NULL} (default), all variables in \code{x}
-#'        that are not in \code{init.model} are considered.
+#'        be performed. If `NULL` (default), all variables in `x` that are not
+#'        in `init.model` are considered.
 #' @param test Type of statistical paired test to use (ignored if
-#'        \code{sel.crit="total.loglik"}).
-#' @param sel.crit Selection criterion: \code{"paired.test"} chooses the
+#'        `sel.crit="total.loglik"`).
+#' @param sel.crit Selection criterion: `"paired.test"` chooses the
 #'        variable with smallest p-value using the paired test specified by
-#'        \code{test} (see \strong{Details}), as long as this is smaller than
-#'        \code{max.pval}; \code{"total.loglik"} picks the variable that gives
-#'        the largest increase in log-likelihood; \code{"both"} attempts to
+#'        `test` (see **Details**), as long as this is smaller than
+#'        `max.pval`; `"total.loglik"` picks the variable that gives
+#'        the largest increase in log-likelihood; `"both"` attempts to
 #'        combine both previous criteria, choosing the variable that produces
 #'        the largest increase in log-likelihood only among the best 5
 #'        variables ranked according to the paired-test p-value.
 #' @param num.filter Number of variables to be retained by the univariate
-#'        association filter (see \strong{Details}), which can only be enabled
-#'        if \code{family=binomial()}. Variables listed in \code{init.model}
+#'        association filter (see **Details**), which can only be enabled
+#'        if `family=binomial()`. Variables listed in `init.model`
 #'        are never filtered. If set to 0 (default), the filter is disabled.
 #' @param filter.ignore Vector of variable names that should not be pruned by
 #'        the univariate association filter so that they are always allowed to
-#'        be selected (ignored if \code{num.filter=0}).
+#'        be selected (ignored if `num.filter=0`).
 #' @param num.inner.folds Number of folds in the inner cross-validation. It
 #'        must be at least 5 (default: 30).
-#' @param max.iters Maximum number of iterations (default: 15).
+#' @param max.iters Maximum number of iterations (default: 10).
+#' @param min.llk.diff Minimum improvement in log-likelihood required before
+#'        selection is terminated (default: 2).
 #' @param max.pval Interrupt the selection when the best achievable p-value
 #'        exceeds this threshold (default: 0.5).
-#' @param min.llk.diff Interrupt the selection when the best achievable
-#'        improvement in log-likelihood is smaller than this threshold
-#'        (default: 0).
 #' @param seed Seed of the random number generator for the inner folds.
 #' @param verbose Whether the variable chosen at each iteration should be
-#'        printed out (default: \code{TRUE}).
+#'        printed out (default: `TRUE`).
 #'
 #' @return
-#' An object of class \code{fs} containing the following fields:
-#' \describe{
-#' \item{fs:}{A dataframe containing the forward selection summary.}
-#' \item{init:}{The set of variables used in the initial model.}
-#' \item{panel:}{ Names of variables selected (in order).}
-#' \item{init.model:}{Right-hand side of the formula corresponding to the
+#' An object of class `fs` containing the following fields:
+#' \item{fs}{A dataframe containing the forward selection summary.}
+#' \item{init}{The set of variables used in the initial model.}
+#' \item{panel}{Names of variables selected (in order).}
+#' \item{init.model}{Right-hand side of the formula corresponding to the
 #'       initial model.}
-#' \item{final.model:}{Right-hand side of the formula corresponding to the
+#' \item{final.model}{Right-hand side of the formula corresponding to the
 #'       final model after forward selection.}
-#' \item{family:}{Type of model fitted.}
-#' \item{params:}{List of parameters used.}
-#' \item{iter1:}{Summary statistics for all variables at the first iteration.}
-#' \item{all.iter:}{Validation log-likelihoods for all inner folds at all
+#' \item{family}{Type of model fitted.}
+#' \item{params}{List of parameters used.}
+#' \item{iter1}{Summary statistics for all variables at the first iteration.}
+#' \item{all.iter}{Validation log-likelihoods for all inner folds at all
 #'       iterations.}
-#' }
 #'
 #' @examples
-#' # register a parallel cluster with two cores
-#' library(doParallel)
-#' registerDoParallel(2)
-#'
+#' \dontshow{oldopts <- options(mc.cores=2)}
 #' data(diabetes)
-#' fs.res <- forward.selection(X.diab, Y.diab, ~ age + sex, family=gaussian(),
-#'                             choose.from=1:10, num.inner.folds=5, max.iters=3)
+#' fs.res <- fs(Y ~ age + sex, data=diabetes, family=gaussian(),
+#'              choose.from=1:10, num.inner.folds=5, max.iters=3)
 #' summary(fs.res)
+#' \dontshow{options(oldopts)}
 #'
-#' # close the parallel cluster
-#' stopImplicitCluster()
-#'
-#' @seealso \code{\link{nested.forward.selection}} and \code{\link{summary.fs}}.
+#' @rdname forward.selection
+#' @seealso [nested.fs()] and [summary.fs()].
 #' @keywords multivariate
-#' @importFrom foreach foreach %dopar%
-#' @importFrom stats coefficients glm predict t.test update wilcox.test
-#' @importFrom utils head tail
 #' @export
-forward.selection <- function(x, y, init.model, family,
-                              choose.from=NULL, test=c("t", "wilcoxon"),
-                              sel.crit=c("paired.test", "total.loglik", "both"),
-                              num.filter=0, filter.ignore=NULL,
-                              num.inner.folds=30, max.iters=15, max.pval=0.5,
-                              min.llk.diff=0, seed=50, verbose=TRUE) {
-  univ.glm <- function(model, xy.train, xy.test) {
-    regr <- glm(model, data=xy.train, family=family)
-    y.pred <- predict(regr, newdata=xy.test, type="response")
-    y.test <- xy.test$nestfs_y_
-    loglik <- loglikelihood(family, y.test, y.pred, summary(regr)$dispersion)
-    res <- cbind(coefficients(summary(regr))[, c(1, 4), drop=FALSE], loglik)
+fs <- function(formula, data, family, choose.from=NULL, test=c("t", "wilcoxon"),
+               num.inner.folds=30, max.iters=10, min.llk.diff=2, max.pval=0.5,
+               sel.crit=c("paired.test", "total.loglik", "both"),
+               num.filter=0, filter.ignore=NULL, seed=50, verbose=TRUE) {
+  univ.glm <- function(formula, data, idx.test, family) {
+    res <- glm.inner(formula, data, idx.test, family, store.glm=TRUE)
+    res <- cbind(coef(summary(res$regr))[, c(1, 4), drop=FALSE], res$test.llk)
     colnames(res) <- c("coef", "p.value", "valid.llk")
     return(res)
   }
@@ -144,10 +145,9 @@ forward.selection <- function(x, y, init.model, family,
   }
 
   ## argument checks
-  if (nrow(x) != length(y))
-    stop("Mismatched dimensions.")
-  y <- validate.outcome(y)
-  choose.from <- validate.choose.from(choose.from, x)
+  y <- validate.model.outcome(formula, data)
+  x <- as.data.frame(data)
+  init.model <- as.formula(formula)
   family <- validate.family(family, y)
   test <- match.arg(test)
   pval.test <- list(t=t.test, wilcoxon=wilcox.test)[[test]]
@@ -161,20 +161,18 @@ forward.selection <- function(x, y, init.model, family,
   if (min.llk.diff < 0)
     stop("min.llk.diff cannot be negative.")
 
-  ## set up initial model
-  init.model <- validate.init.model(init.model)
-  init.vars <- setdiff(all.vars(init.model), "nestfs_y_")
-
-  ## check that all variables exist in the dataframe of predictors
-  var.match <- match(init.vars, colnames(x))
-  if (anyNA(var.match))
-    stop("'", paste(init.vars[is.na(var.match)], collapse="', '"),
-         "' not present in x.")
-
   ## check that there is no missingness in the initial model
+  init.vars <- all.vars(init.model)
   if (anyNA(x[, init.vars]))
     stop("Missing values in the variables of the initial model.")
 
+  ## remove variables in the initial model from choose.from
+  choose.from <- validate.choose.from(choose.from, x)
+  choose.from <- setdiff(choose.from, match(init.vars, colnames(x)))
+
+  ## remove outcome from the initial set of variables
+  init.vars <- setdiff(init.vars, as.character(init.model)[2])
+
   ## create the inner folds
   folds <- create.folds(num.inner.folds, nrow(x), seed=seed)
 
@@ -185,12 +183,21 @@ forward.selection <- function(x, y, init.model, family,
     model.vars <- "<empty>"
   model.llks <- c(rep(NA, num.init.vars - 1), 0)
   model.pvals <- model.iter <- rep(NA, num.init.vars)
+
+  ## expand the interaction terms
+  init.model <- update(init.model, . ~ .)
   model <- init.model
 
   ## limit the number of variables to choose from
   keep.vars <- union(choose.from, match(init.vars, colnames(x)))
   x <- x[, keep.vars]
 
+  ## initialize the parallel cluster
+  if (.Platform$OS.type == "windows") {
+    cl <- parallel::makePSOCKcluster(getOption("mc.cores", 1))
+    on.exit(parallel::stopCluster(cl))
+  }
+
   ## filtering according to association with outcome
   if (num.filter > 0) {
 
@@ -202,10 +209,8 @@ forward.selection <- function(x, y, init.model, family,
       stop("filter.ignore should be a character vector or NULL.")
 
     ## run the filter on the training part of all inner folds
-    fold <- NULL   # silence a note raised by R CMD check
     ignore.vars <- c(init.vars, filter.ignore)
-    all.filt.idx <- foreach(fold=1:num.inner.folds, .combine=c) %dopar% {
-
+    par.fun <- function(fold) {
       train.idx <- setdiff(seq(nrow(x)), folds[[fold]])
       x.train <- x[train.idx, ]
       y.train <- y[train.idx]
@@ -213,8 +218,16 @@ forward.selection <- function(x, y, init.model, family,
                                     ignore=ignore.vars)
     }
 
+    if (.Platform$OS.type != "windows") {
+      all.filt.idx <- parallel::mclapply(X=1:num.inner.folds, FUN=par.fun,
+                                         mc.preschedule=FALSE)
+    } else { # windows
+      all.filt.idx <- parallel::parLapply(X=1:num.inner.folds, cl=cl,
+                                          fun=par.fun)
+    }
+
     ## keep the union of the variables retained in the inner folds
-    filt.idx <- sort(table(all.filt.idx), decreasing=TRUE)
+    filt.idx <- sort(table(unlist(all.filt.idx)), decreasing=TRUE)
     filt.idx <- as.integer(names(filt.idx))[1:num.filter]
     keep.idx <- union(match(ignore.vars, colnames(x)), filt.idx)
     keep.idx <- keep.idx[!is.na(keep.idx)]
@@ -222,7 +235,7 @@ forward.selection <- function(x, y, init.model, family,
   }
 
   ## variable selection
-  all.vars <- colnames(x)
+  all.vars <- setdiff(colnames(x), as.character(init.model)[2])
   all.iter <- list()
   iter1 <- NULL
 
@@ -233,27 +246,31 @@ forward.selection <- function(x, y, init.model, family,
       break
 
     ## loop over the folds
-    res.inner <- foreach(fold=1:num.inner.folds) %dopar% {
-
+    par.fun <- function(fold) {
       test.idx <- folds[[fold]]
-      train.idx <- setdiff(seq(nrow(x)), test.idx)
-      xy.train <- cbind(nestfs_y_=y[train.idx], x[train.idx, ])
-      xy.test <- cbind(nestfs_y_=y[test.idx], x[test.idx, ])
 
       ## current model
-      curr <- univ.glm(model, xy.train, xy.test)
-      all.stats <- tail(curr, n=1)
+      curr <- univ.glm(model, data, test.idx, family)
+      all.stats <- utils::tail(curr, n=1)
 
       ## models augmented with one additional variable at a time
       for (var in other.vars) {
-        model.augm <- update(model, paste(". ~ .", var, sep=" + "))
-        augm <- univ.glm(model.augm, xy.train, xy.test)
-        all.stats <- rbind(all.stats, tail(augm, n=1))
+        model.augm <- update(model, reformulate(c(".", var)))
+        augm <- univ.glm(model.augm, data, test.idx, family)
+        all.stats <- rbind(all.stats, utils::tail(augm, n=1))
       }
       rownames(all.stats) <- c("Base", other.vars)
       return(all.stats)
     }
 
+    if (.Platform$OS.type != "windows") {
+      res.inner <- parallel::mclapply(X=1:num.inner.folds, FUN=par.fun,
+                                      mc.preschedule=FALSE)
+    } else { # windows
+      res.inner <- parallel::parLapply(X=1:num.inner.folds, cl=cl,
+                                       fun=par.fun)
+    }
+
     ## collect all validation log-likelihoods
     all.llk <- sapply(res.inner, function(z) z[, "valid.llk"])
     all.iter[[iter]] <- all.llk
@@ -308,7 +325,7 @@ forward.selection <- function(x, y, init.model, family,
     model.pvals <- c(model.pvals, chosen.pval)
     model.llks <- c(model.llks, chosen.llk)
     model.iter <- c(model.iter, rep(iter, length(chosen.var)))
-    model <- update(model, paste(". ~ .", chosen.var, sep=" + "))
+    model <- update(model, reformulate(c(".", chosen.var)))
   }
 
   res <- list(fs=data.frame(vars=model.vars, fdr=model.pvals, llks=model.llks,
@@ -326,7 +343,33 @@ forward.selection <- function(x, y, init.model, family,
   return(res)
 }
 
-#' Run nested forward selection
+#' @inherit fs
+#' @template args-forward
+#' @template args-outcome
+#' @param ... Further arguments to `fs`.
+#'
+#' @details
+#' `forward.selection` provides the legacy interface used up to version 0.9.2.
+#' It is considered discontinued, and in the future it will be deprecated and
+#' eventually removed.
+#'
+#' @export
+forward.selection <- function(x, y, init.model, family, ...) {
+  if (nrow(x) != length(y))
+    stop("Mismatched dimensions.")
+  init.model <- validate.init.model(init.model)
+
+  ## check that all variables exist in the dataframe of predictors
+  init.vars <- setdiff(all.vars(init.model), "nestfs_y_")
+  var.match <- match(init.vars, colnames(x))
+  if (anyNA(var.match))
+    stop("'", paste(init.vars[is.na(var.match)], collapse="', '"),
+         "' not present in x.")
+
+  fs(init.model, cbind(x, nestfs_y_=y), family, ...)
+}
+
+#' Nested cross-validated forward selection
 #'
 #' Run nested forward selection starting from a set of variables or a model.
 #'
@@ -334,51 +377,41 @@ forward.selection <- function(x, y, init.model, family,
 #' of the selected panels on withdrawn data by running forward selection on
 #' a predetermined set of folds.
 #'
-#' @template args-forward
-#' @template args-outcome
+#' @template args-interface
 #' @template args-family
 #' @template args-folds
-#' @param ... Arguments to \code{forward.selection}.
+#' @param ... Arguments to [fs()].
 #'
 #' @return
-#' An object of class \code{nestfs} of length equal to
-#' \code{length(folds)}, where each element is an object of class
-#' \code{fs} containing the following additional fields:
-#' \describe{
-#' \item{fit:}{Predicted values for the withdrawn observations.}
-#' \item{obs:}{Observed values for the withdrawn observations.}
-#' \item{test.idx:}{Indices of the the withdrawn observations for this fold.}
-#' \item{model:}{Summary of the model built using the selected panel.}
-#' }
+#' An object of class `nestfs` of length equal to `length(folds)`, where each
+#' element is an object of class `fs` containing the following additional fields:
+#' \item{fit}{Predicted values for the withdrawn observations.}
+#' \item{obs}{Observed values for the withdrawn observations.}
+#' \item{test.idx}{Indices of the the withdrawn observations for this fold.}
+#' \item{model}{Summary of the model built using the selected panel.}
 #'
 #' @examples
-#' # register a parallel cluster with two cores
-#' library(doParallel)
-#' registerDoParallel(2)
-#'
+#' \dontshow{oldopts <- options(mc.cores=2)}
 #' data(diabetes)
-#' folds <- create.folds(2, nrow(X.diab), seed=1)
-#' nestfs.res <- nested.forward.selection(X.diab, Y.diab, ~ age + sex,
-#'                                        gaussian(), folds, choose.from=1:10,
-#'                                        num.inner.folds=5, max.iters=3)
+#' folds <- create.folds(2, nrow(diabetes), seed=1)
+#' nestfs.res <- nested.fs(Y ~ age + sex, diabetes, gaussian(), folds,
+#'                         choose.from=1:10, num.inner.folds=5, max.iters=3)
 #' summary(nestfs.res)
+#' \dontshow{options(oldopts)}
 #'
-#' # close the parallel cluster
-#' stopImplicitCluster()
-#'
+#' @rdname nested.forward.selection
 #' @seealso
-#' \code{\link{forward.selection}}, \code{\link{summary.nestfs}} and
-#' \code{\link{nested.performance}}.
+#' [fs()], [summary.nestfs()] and [nested.performance()].
 #' @keywords multivariate
 #' @export
-nested.forward.selection <- function(x, y, init.model, family, folds, ...) {
-
-  res <- list()
-  y <- validate.outcome(y)
+nested.fs <- function(formula, data, family, folds, ...) {
+  y <- validate.model.outcome(formula, data)
+  x <- as.data.frame(data)
   family <- validate.family(family, y)
   folds <- validate.folds(folds, x)
   verbose <- isTRUE(list(...)$verbose) || is.null(list(...)$verbose)
 
+  res <- list()
   num.folds <- length(folds)
   for (fold in 1:num.folds) {
 
@@ -387,15 +420,11 @@ nested.forward.selection <- function(x, y, init.model, family, folds, ...) {
           "-", format(Sys.time(), "%H:%M"), "\n")
 
     test.idx <- folds[[fold]]
-    train.idx <- setdiff(seq(nrow(x)), test.idx)
-    x.train <- x[train.idx, ]; y.train <- y[train.idx]
-
-    fs <- forward.selection(x.train, y.train, init.model, family, ...)
+    fs <- fs(formula, x[-test.idx, ], family, ...)
 
     ## fit the final model on the training set and evaluate it on the test set
-    model <- validate.init.model(fs$final.model)
-    model <- glm.inner(x, y, model, test.idx, family)
-    stopifnot(all.equal(model$obs, y[test.idx]))
+    model <- update(formula, reformulate(c(".", fs$final.model)))
+    model <- glm.inner(model, x, test.idx, family)
 
     ## extract the model coefficients and report them in the forward selection
     ## object.
@@ -411,7 +440,7 @@ nested.forward.selection <- function(x, y, init.model, family, folds, ...) {
     idx.coefs <- pmatch(panel, rownames(model$summary))
     fs$fs$coef[match(panel, fs$fs$vars)] <- model$summary[idx.coefs, "Estimate"]
     fs$fit <- model$fit
-    fs$obs <- model$obs
+    fs$obs <- unname(model$obs)
     fs$test.idx <- test.idx
     fs$model <- model$summary
     res[[fold]] <- fs
@@ -420,60 +449,64 @@ nested.forward.selection <- function(x, y, init.model, family, folds, ...) {
   return(res)
 }
 
+#' @inherit nested.fs
+#' @template args-forward
+#' @template args-outcome
+#'
+#' @details
+#' `nested.forward.selection` provides the legacy interface used up to version
+#' 0.9.2. It is considered discontinued, and in the future it will be deprecated
+#' and eventually removed.
+#'
+#' @export
+nested.forward.selection <- function(x, y, init.model, family, folds, ...) {
+  y <- validate.outcome(y)
+  family <- validate.family(family, y)
+  init.model <- validate.init.model(init.model)
+  nested.fs(init.model, cbind(x, nestfs_y_=y), family, folds, ...)
+}
+
 #' Cross-validated generalized linear models
 #'
 #' Run linear or logistic regression on a set of cross-validation folds.
 #'
 #' This can be used to establish a baseline model, often built only on the
-#' initial set of covariates (those that would be passed through the
-#' \code{init.model} argument to \code{forward.selection}).
+#' initial set of covariates.
 #'
-#' @param x Dataframe of predictors.
-#' @template args-outcome
-#' @param model Either a formula or a vector of names for the set of variables
-#'        that define the model to be fitted.
+#' @template args-interface
 #' @template args-family
 #' @template args-folds
-#' @param store.glm Whether the object produced by \code{glm} should be
-#'        stored (default: \code{FALSE}).
+#' @param store.glm Whether the object produced by `glm` should be
+#'        stored (default: `FALSE`).
 #'
 #' @return
-#' An object of class \code{nestglm} of length equal to \code{length(folds)},
+#' An object of class `nestglm` of length equal to `length(folds)`,
 #' where each entry contains the following fields:
-#' \describe{
-#' \item{summary:}{Summary of the coefficients of the model fitted on the
+#' \item{summary}{Summary of the coefficients of the model fitted on the
 #'       training observations.}
-#' \item{family:}{Type of model fitted.}
-#' \item{fit:}{Predicted values for the withdrawn observations.}
-#' \item{obs:}{Observed values for the withdrawn observations.}
-#' \item{test.llk:}{Test log-likelihood.}
-#' \item{test.idx:}{Indices of the the withdrawn observations for this fold.}
-#' \item{regr:}{Object created by glm (only if \code{store.glm=TRUE}).}
-#' }
+#' \item{family}{Type of model fitted.}
+#' \item{fit}{Predicted values for the withdrawn observations.}
+#' \item{obs}{Observed values for the withdrawn observations.}
+#' \item{test.llk}{Test log-likelihood.}
+#' \item{test.idx}{Indices of the the withdrawn observations for this fold.}
+#' \item{regr}{Object created by `glm` (only if `store.glm=TRUE`).}
 #'
 #' @examples
-#' # register a parallel cluster with two cores
-#' library(doParallel)
-#' registerDoParallel(2)
-#'
+#' \dontshow{oldopts <- options(mc.cores=2)}
 #' data(diabetes)
-#' folds <- create.folds(10, nrow(X.diab), seed=1)
-#' base.res <- nested.glm(X.diab, Y.diab, c("age", "sex", "bmi", "map"),
-#'                        gaussian(), folds)
+#' folds <- create.folds(10, nrow(diabetes), seed=1)
+#' res <- nested.glm(Y ~ age + sex + bmi + map, diabetes, gaussian(), folds)
+#' \dontshow{options(oldopts)}
 #'
-#' # close the parallel cluster
-#' stopImplicitCluster()
-#'
-#' @seealso \code{\link{nested.performance}}.
+#' @seealso [nested.performance()].
 #' @keywords multivariate
 #' @export
-nested.glm <- function(x, y, model, family, folds, store.glm=FALSE) {
+nested.glm <- function(formula, data, family, folds, store.glm=FALSE) {
 
   ## argument checks
-  if (nrow(x) != length(y))
-    stop("Mismatched dimensions.")
-  y <- validate.outcome(y)
-  model <- validate.init.model(model)
+  y <- validate.model.outcome(formula, data)
+  x <- as.data.frame(data)
+  model <- as.formula(formula)
   family <- validate.family(family, y)
   folds <- validate.folds(folds, x)
 
@@ -484,7 +517,7 @@ nested.glm <- function(x, y, model, family, folds, store.glm=FALSE) {
     stop("'", paste(vars[is.na(var.match)], collapse="', '"),
          "' not present in x.")
 
-  res <- lapply(folds, function(z) glm.inner(x, y, model, z, family, store.glm))
+  res <- lapply(folds, function(z) glm.inner(model, x, z, family, store.glm))
   class(res) <- c("nestglm")
   return(res)
 }
@@ -495,24 +528,21 @@ nested.glm <- function(x, y, model, family, folds, store.glm=FALSE) {
 #' Fit a model using all predictors provided on the training observations, then
 #' test it on the withdrawn observations.
 #'
-#' @param x Dataframe of predictors.
+#' @template args-interface
 #' @template args-outcome
-#' @param model Formula for the model to be fitted.
 #' @param idx.test Indices of observations to withdraw.
 #' @template args-family
-#' @param store.glm Whether the object produced by \code{glm} should be
-#'        stored (default: \code{FALSE}).
+#' @param store.glm Whether the object produced by `glm` should be
+#'        stored (default: `FALSE`).
 #'
 #' @return
-#' A list of length equal to \code{length(folds)}.
+#' A list with the results of fitting a model on a specific fold.
 #'
-#' @importFrom stats as.formula glm predict
 #' @noRd
-glm.inner <- function(x, y, model, idx.test, family, store.glm=FALSE) {
-  idx.train <- setdiff(1:nrow(x), idx.test)
-  regr <- glm(model, data=cbind(nestfs_y_=y, x)[idx.train, ], family=family)
-  y.pred <- predict(regr, newdata=x[idx.test, ], type="response")
-  y.test <- y[idx.test]
+glm.inner <- function(formula, data, idx.test, family, store.glm=FALSE) {
+  regr <- glm(formula, data=data[-idx.test, ], family=family)
+  y.pred <- predict(regr, newdata=data[idx.test, ], type="response")
+  y.test <- validate.model.outcome(formula, data[idx.test, ])
   loglik <- loglikelihood(family, y.test, y.pred, summary(regr)$dispersion)
   res <- list(summary=coefficients(summary(regr)), family=family$family,
               fit=y.pred, obs=y.test, test.llk=loglik, test.idx=idx.test)
@@ -533,168 +563,3 @@ glm.inner <- function(x, y, model, idx.test, family, store.glm=FALSE) {
 loglikelihood <- function(family, obs, fit, disp) {
   sum(family$dev.resids(obs, fit, -0.5 / disp) - log(disp) / 2)
 }
-
-#' Validate the outcome variable
-#'
-#' Ensure that the outcome variable has been specified correctly.
-#'
-#' @param y Outcome variable to test.
-#'
-#' @return
-#' A valid outcome variable. The function throws an error if the outcome
-#' variable cannot be used.
-#'
-#' @noRd
-validate.outcome <- function(y) {
-  if (anyNA(y))
-    stop("Outcome variable contains missing values.", call.=FALSE)
-  if (is.character(y))
-    stop("Outcome variable cannot be a character vector.", call.=FALSE)
-  if (is.factor(y)) {
-    if (length(levels(y)) != 2)
-      stop("A factor outcome variable can only have two levels.", call.=FALSE)
-    y <- as.integer(y) - 1
-  }
-  if (!(is.numeric(y) || is.logical(y)))
-    stop("Outcome variable of invalid type.", call.=FALSE)
-
-  return(as.numeric(y))
-}
-
-#' Validate initial model
-#'
-#' Ensure that the initial model has been specified correctly.
-#'
-#' @param model Model definition to test.
-#'
-#' @return
-#' A formula describing the initial model. The function throws an error if the
-#' model parameter cannot be used.
-#'
-#' @importFrom methods is
-#' @importFrom stats as.formula update
-#' @noRd
-validate.init.model <- function(model) {
-  if (is.null(model) || length(model) == 0) {
-    model <- y ~ 1
-  }
-  else if (is.character(model)) {
-    if (any(model == ""))
-      stop("init.model contains an empty string.", call.=FALSE)
-    if (length(model) == 1 && grepl("~", model))
-      model <- as.formula(model)
-    else
-      model <- as.formula(paste("y ~", paste(model, collapse= " + ")))
-  }
-  else if (!is(model, "formula"))
-    stop("init.model specified incorrectly.", call.=FALSE)
-
-  ## rename the left-hand side or add it if not present
-  model <- update(model, "nestfs_y_ ~ .")
-
-  return(model)
-}
-
-#' Validate the family argument
-#'
-#' Ensure that the family argument has been specified correctly.
-#' This is inspired by code in \code{\link{glm}}.
-#'
-#' @param family Family argument to test.
-#' @param y Outcome variable.
-#'
-#' @return
-#' A valid family. The function throws an error if the family argument cannot
-#' be used.
-#'
-#' @importFrom methods is
-#' @noRd
-validate.family <- function(family, y) {
-  if (missing(family))
-    stop("Argument of 'family' is missing.", call.=FALSE)
-  if (is.character(family))
-    tryCatch(
-      family <- get(family, mode="function", envir=parent.frame(2)),
-      error=function(e)
-        stop("'", family, "' is not a valid family.", call.=FALSE)
-    )
-  if (is.function(family))
-    family <- family()
-  if (!is(family, "family"))
-    stop("Argument of 'family' is not a valid family.", call.=FALSE)
-  if (!family$family %in% c("gaussian", "binomial"))
-    stop("Only 'gaussian' and 'binomial' are supported families.", call.=FALSE)
-
-  if (family$family == "binomial") {
-    if (length(table(y)) != 2)
-      stop("y must contain two classes with family=binomial().", call.=FALSE)
-    if (!is.factor(y) && any(y < 0 | y > 1))
-      stop("y must contain 0-1 values with family=binomial().", call.=FALSE)
-  }
-
-  return(family)
-}
-
-#' Validate the choose.from argument
-#'
-#' Ensure that the \code{choose.from} argument has been specified correctly.
-#'
-#' @param choose.from Argument to test.
-#' @param x Dataframe of predictors.
-#'
-#' @return
-#' A valid vector of variable indices. The function throws an error if the
-#' argument cannot be used.
-#'
-#' @noRd
-validate.choose.from <- function(choose.from, x) {
-  if (is.null(choose.from))
-    choose.from <- seq(ncol(x))
-  else {
-    if (is.numeric(choose.from)) {
-      if (anyNA(choose.from))
-        stop("choose.from contains missing values.", call.=FALSE)
-      if (length(choose.from) > 0 &&
-          (min(choose.from) < 1 || max(choose.from) > ncol(x)))
-        stop("choose.from contains out of bounds indices.", call.=FALSE)
-      if (any(choose.from != as.integer(choose.from)))
-        stop("choose.from contains floating point values.", call.=FALSE)
-    }
-    else if (is.character(choose.from)) {
-      choose.from <- match(choose.from, colnames(x))
-      if (anyNA(choose.from))
-        stop("choose.from contains names that cannot be matched.", call.=FALSE)
-    }
-    else
-      stop("choose.from should be an integer or character vector.", call.=FALSE)
-  }
-  return(as.integer(choose.from))
-}
-
-#' Validate the folds argument
-#'
-#' Ensure that the \code{folds} argument has been specified correctly.
-#'
-#' @param folds Argument to test.
-#'
-#' @return
-#' A valid list of folds. The function throws an error if the argument cannot
-#' be used.
-#'
-#' @noRd
-validate.folds <- function(folds, x) {
-  if (!is.list(folds))
-    stop("folds expected to be a list.", call.=FALSE)
-  all.idx <- unlist(folds)
-  if (anyNA(all.idx))
-    stop("folds contains missing values.", call.=FALSE)
-  if (!is.numeric(all.idx))
-    stop("folds contains non-numerical values.", call.=FALSE)
-  if (any(all.idx != as.integer(all.idx)))
-    stop("folds contains non-integer values", call.=FALSE)
-  if (any(table(all.idx) > 1))
-    stop("folds contains repeated indices.", call.=FALSE)
-  if (any(all.idx <= 0 | all.idx > nrow(x)))
-    stop("folds contains out of bounds indices.", call.=FALSE)
-  return(folds)
-}
